/*************************************************************************

sscsss.h - declarations used in multiple files

This file is a part of sscsss -- Simple Single Cycle Sample Set Synthesizer.

Copyright (C) 2002 David Baker

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*************************************************************************/

#ifndef _SSCSSS_H
#define _SSCSSS_H

#include <stdio.h>			/* for FILE typedef		    */

/* pointers to structures */

typedef struct m_s		*M_P;	/* to main structure		    */

typedef struct wave_set_s	*WS_P;  /* to wave set structure	    */
typedef struct offset_s		*O_P;	/* to offset structure  	    */
typedef struct filter_s		*FI_P;	/* to filter structure  	    */
typedef struct scale_s		*SC_P;	/* to scale structure   	    */
typedef struct gpoint_s		*GP_P;	/* to gpoint structure		    */
typedef struct formant_s	*FO_P;	/* to formant structure		    */
typedef struct wave_split_s	*WP_P;  /* to wave split structure	    */
typedef struct note_list_s	*N_P;	/* to note list			    */
typedef struct wave_split_c_s	*WPC_P; /* to wave split condition struct   */

typedef struct wave_s		*W_P;	/* to wave structure		    */
typedef struct sample_s		*S_P;	/* to sample structure		    */
typedef struct harmonic_s	*H_P;	/* to harmonic structure	    */

/* structures */

typedef struct m_s {	/* main structure				    */
    FILE	*infile;	/* file handle input			    */
    FILE	*outfile;	/* file handle output			    */
    FILE	*log;		/* file handle log			    */
    int		state;		/* input state				    */
    int		linecnt;	/* input line number			    */
    int		f_waves;	/* file total number of waves		    */
    int		f_samples;	/* file total number of samples		    */
    int		f_size;		/* file total bytes in samples		    */
    char	v_cat[3];	/* voice category			    */
    char	v_name[13];	/* voice name				    */
    int 	v_bend;		/* voice max upward pitch bend (half notes) */
    int		v_low;		/* voice low note			    */
    int		v_high;		/* voice high note			    */
    int		v_norm;		/* voice normalization type		    */
    double	o_lowfreq;	/* option low frequency			    */
    double	o_mindba;	/* option minimal dba value to generate	    */
    int		o_debug;	/* option debug				    */
    int		o_logws;	/* option log waves from wave sets	    */
    int		o_mkplot;	/* option make gnuplot file for waves	    */
    int		o_plotfreq;	/* option plot frequency		    */
    int		o_plotpause;	/* option pause command at end of plot file */
    int		d_malloc;	/* count of malloc calls (except samples)   */
    int		d_free;		/* count of free calls			    */
    WS_P	ws_list;	/* wave set list			    */
    W_P		w_list;		/* wave	list				    */
    /* input temps, initialize to 0, handle resets			    */
    int		v_waves;	/* number of waves in current voice	    */
    int		w_samples;	/* number of samples in current wave	    */
    int		h_no;		/* harmonic number in current sample	    */
    int		s_type;		/* code for parameter being scaled	    */
    /* input temps, initialize to NULL and reset when list is complete	    */
    WS_P	ws_back;	/* back of current set list		    */
    O_P		o_back;		/* back of current offset list		    */
    FI_P	fi_back;	/* back of current filter list		    */
    SC_P	sc_back;	/* back of current scale list		    */
    GP_P	gp_back;	/* back of current graph point list	    */
    FO_P	fo_back;	/* back of current formant list		    */
    WP_P	wp_back;	/* back of current split list		    */
    N_P		n_back;		/* back of current note list		    */
    WPC_P	wpc_back;	/* back of current condition list	    */
    W_P		w_back;		/* back of current wave list		    */
    S_P		s_back;		/* back of current sample list		    */
    H_P		h_back;		/* back of current harmonic list	    */
} M_S;

typedef struct wave_set_s { /* wave set structure			    */
    O_P		o_list;		/* list of offsets                          */
    FI_P	fi_list;	/* list of filters			    */
    FO_P	fo_list;	/* list of formants			    */
    WP_P	wp_list;	/* list of wave splits			    */
    WS_P	next;		/* next wave set                            */
} WS_S;

typedef struct offset_s { /* offset structure				    */
    int		no;		/* harmonic number                          */
    double	level;		/* level offset				    */
    SC_P	level_s_list;	/* level scaling list			    */
    double	phase;		/* phase offset (from 0)		    */
    SC_P	phase_s_list;	/* phase scaling list			    */
    O_P		next;		/* next offset                              */
} O_S;

typedef struct filter_s { /* filter structure				    */
    int		type;		/* filter type				    */
    double	p1;		/* meaning of p? depends on type	    */
    SC_P	p1_s_list;
    double	p2;
    SC_P	p2_s_list;
    double	p3;
    SC_P	p3_s_list;
    double	p4;
    SC_P	p4_s_list;
    GP_P	gp_list;
    FI_P	next;		/* next filter				    */
} FI_S;

typedef struct scale_s { /* scaling structure				    */
    double	value;		/* scale to this value			    */
    double	freq;		/*   at this frequency			    */
    SC_P	next;		/* next scale record			    */
} SC_S;

typedef struct gpoint_s { /* graphic filter/formant point structure	    */
    double	freq;
    double	gain;
    GP_P	next;
} GP_S;

typedef struct formant_s { /* formant structure				    */
    int		type;		/* formant type				    */
    double	p1;		/* meaning of p? depends on type	    */
    double	p2;
    double	p3;
    double	p4;
    GP_P	gp_list;
    FO_P	next;		/* next formant				    */
} FO_S;

typedef struct wave_split_s { /* wave split structure			    */
    int		notes;		/* number of samples to generate	    */
    N_P		note_list;	/* note list				    */
    WPC_P	wpc_list;	/* list of conditions			    */
    WP_P	next;		/* next split				    */
} WP_S;

typedef struct note_list_s { /* note list structure			    */
    int		center;		/* center note (MIDI #)			    */
    int		low;		/* low note (MIDI #)			    */
    int		high;		/* high note (MIDI #)			    */
    N_P		next;		/* next note				    */
} N_S;

typedef struct wave_split_c_s {/* wave split condition structure	    */
    int		rel;		/* arithmetic relation 			    */
    int		harmonic;	/* harmonic number for comparison	    */
    double	freq;		/* frequency for comparisons		    */
    int		divisor;	/* divisor for modulus			    */
    int		remainder;	/* remainder for modulus		    */
    WPC_P	next;		/* next split				    */
} WPC_S;

typedef struct wave_s {	/* wave structure				    */
    int		no;		/* wave number in file			    */
    int		layers;		/* number of layers (samples)		    */
    char	cat[3];		/* wave category			    */
    char	name[9];	/* wave name				    */
    S_P		s_list;		/* sample list				    */
    W_P		next;		/* next wave				    */
} W_S;

typedef struct sample_s { /* sample structure				    */
    int		no;		/* sample number in file		    */
    char	cat[3];		/* sample category			    */
    char	name[9];	/* sample name				    */
    int		center;		/* sample center note			    */
    int		low;		/* layer low note			    */
    int		high;		/* layer high note			    */
    double	freq;		/* sample frequency			    */
    int		loop_len;	/* points in loop in ex, calc.              */
    int		loop_fract;	/* fraction for loop in ex, calc.           */
    int		smpl_len;	/* points in sample in ex, calc.            */
    double	max_absv;	/* maximum absolute value for sample points */
    double	*data;		/* pointer to sample data		    */
    H_P		h_list;		/* harmonic list			    */
    S_P		next;		/* next sample				    */
} S_S;

typedef struct harmonic_s { /* harmonic structure			    */
    int		no;		/* harmonic number			    */
    double	level;		/* harmonic level			    */
    double	phase;		/* harmonic phase			    */
    H_P		next;		/* next harmonic			    */
} H_S;

/* enumerations */

enum i_states {	/* input states for input processing state machine */
    IS_INIT,
    IS_FILE,
    IS_OPTION,
    IS_VOICE,
    IS_SET,
    IS_OFFSET,
    IS_OFFSET_SC,
    IS_OFFSET_SC_P,
    IS_FILTER,
    IS_FILTER_SC,
    IS_FILTER_SC_P,
    IS_FILTER_GP,
    IS_FORMANT,
    IS_FORMANT_GP,
    IS_SPLIT,
    IS_NOTE,
    IS_SPLIT_C,
    IS_WAVE,
    IS_SAMPLE,
    IS_HARMONIC,
    IS_EOF
};

enum sc_types {	/* scaling parameters */
    SC_NONE,
    SC_O_LEVEL,
    SC_O_PHASE,
    SC_F_P1,
    SC_F_P2,
    SC_F_P3,
    SC_F_P4
};

enum sp_rels {	/* wave split condition operators */
    SP_C_EQ,
    SP_C_LT,
    SP_C_LE,
    SP_C_GT,
    SP_C_GE,
    SP_C_MOD,
    SP_C_NOT,
    SP_C_AND,
    SP_C_OR
};

enum f_types {	/* filter/formant types */
    F_COS,
    F_TRI,
    F_HPF,
    F_LPF,
    F_LPF_F,
    F_HS,
    F_LS,
    F_BAND,
    F_GRAPH
};

#endif

/* end of sscsss.h */

